package com.jiuqi.syntax;

import java.util.Iterator;
import java.util.Map;
import java.util.HashMap;

public final class DataCell
{
    public static int DC_NEW_COUNT;
    public static int DC_LIVE_COUNT;
    public static final int NONE_KIND = 0;
    public static final int YEAR_KIND = 1;
    public static final int HALFYEAR_KIND = 2;
    public static final int SEASON_KIND = 3;
    public static final int MONTH_KIND = 4;
    public static final int TENDAY_KIND = 5;
    public static final int WEEK_KIND = 6;
    public static final int DAY_KIND = 7;
    public static final int CUSTOM_KIND = 8;
    public static final int GRID_ROW_KIND = 9;
    public static final int GRID_COL_KIND = 10;
    public static final String[] KindSigns;
    public static final String[] KindChinaSigns;
    public static final int MAX_TERM_VALUE = 9999;
    public static final int SUCC_TERM_BASE = 10000;
    public static final int STAT_NONE = 0;
    public static final int STAT_SUM = 1;
    public static final int STAT_COUNT = 2;
    public static final int STAT_MAX = 3;
    public static final int STAT_MIN = 4;
    public static final int STAT_AVG = 5;
    public static final int STAT_FIRST = 6;
    public static final int STAT_LAST = 7;
    public static final int STAT_LJ = 8;
    public static final int STAT_XJHZ = 10;
    public static final int STAT_AUTOSUM = 9;
    public static final String[] StatSigns;
    public String taskName;
    public int year;
    public int termKind;
    public int term;
    public boolean isZB;
    public String zbName;
    public String tableName;
    public int row;
    public int col;
    public boolean fetchMean;
    public boolean hasCond;
    public String condition;
    public HashMap dimens;
    public Object dimenInfos;
    public int statMode;
    public Object statItem;
    public int tableOffset;
    public int offset;
    public int dimSeq;
    
    static {
        DataCell.DC_NEW_COUNT = 0;
        DataCell.DC_LIVE_COUNT = 0;
        KindSigns = new String[] { "", "N", "H", "J", "Y", "X", "Z", "R", "B", "行", "栏" };
        KindChinaSigns = new String[] { "", "年", "半年", "季", "月", "旬", "周", "日", "期", "行", "栏" };
        StatSigns = new String[] { "", "SUM", "COUNT", "MAX", "MIN", "AVG", "FIRST", "LAST", "LJ", "AUTOSUM", "XJHZ" };
    }
    
    public static int kindSignToInt(final String s) {
        for (int i = 1; i < DataCell.KindSigns.length; ++i) {
            if (s.equalsIgnoreCase(DataCell.KindSigns[i]) || s.equalsIgnoreCase(DataCell.KindChinaSigns[i])) {
                return i;
            }
        }
        return 0;
    }
    
    public static String kindIntToSign(final int k) {
        if (k >= 0 && k < DataCell.KindSigns.length) {
            return DataCell.KindSigns[k];
        }
        return "";
    }
    
    public static String kindIntToChinaSign(final int k) {
        if (k >= 0 && k < DataCell.KindChinaSigns.length) {
            return DataCell.KindChinaSigns[k];
        }
        return "";
    }
    
    public static int statSignToInt(final String s) {
        for (int i = 0; i < DataCell.StatSigns.length; ++i) {
            if (DataCell.StatSigns[i].compareToIgnoreCase(s) == 0) {
                return i;
            }
        }
        return -1;
    }
    
    public static String statIntToSign(final int i) {
        if (i >= 0 && i < DataCell.StatSigns.length) {
            return DataCell.StatSigns[i];
        }
        return "";
    }
    
    public DataCell() {
        this.taskName = "";
        this.year = 0;
        this.termKind = 0;
        this.term = 0;
        this.isZB = true;
        this.zbName = "";
        this.tableName = "";
        this.row = 0;
        this.col = 0;
        this.fetchMean = false;
        this.hasCond = false;
        this.condition = "";
        this.dimens = null;
        this.dimenInfos = null;
        this.statItem = null;
        this.tableOffset = 0;
        this.offset = 0;
        this.dimSeq = -1;
        ++DataCell.DC_NEW_COUNT;
        ++DataCell.DC_LIVE_COUNT;
    }
    
    public void init() {
        this.taskName = "";
        this.year = 0;
        this.termKind = 0;
        this.term = 0;
        this.isZB = true;
        this.zbName = "";
        this.tableName = "";
        this.row = 0;
        this.col = 0;
        this.fetchMean = false;
        this.hasCond = false;
        this.condition = "";
        if (this.dimens != null) {
            this.dimens.clear();
        }
        this.statMode = 0;
        this.dimenInfos = null;
        this.statItem = null;
        this.tableOffset = 0;
        this.offset = 0;
    }
    
    public void assign(final DataCell source) {
        this.taskName = source.taskName;
        this.year = source.year;
        this.termKind = source.termKind;
        this.term = source.term;
        this.isZB = source.isZB;
        this.zbName = source.zbName;
        this.tableName = source.tableName;
        this.row = source.row;
        this.col = source.col;
        this.fetchMean = source.fetchMean;
        this.hasCond = source.hasCond;
        this.condition = source.condition;
        if (this.dimens != null) {
            this.dimens.clear();
        }
        if (source.dimens != null) {
            if (this.dimens == null) {
                this.dimens = new HashMap(source.dimens.size());
            }
            this.dimens.putAll(source.dimens);
        }
        this.dimenInfos = source.dimenInfos;
        this.statItem = source.statItem;
        this.statMode = source.statMode;
        this.tableOffset = source.tableOffset;
        this.offset = source.offset;
        this.dimSeq = source.dimSeq;
    }
    
    @Override
    public String toString() {
        final StringBuffer stringBuffer = new StringBuffer();
        if (this.isZB) {
            stringBuffer.append("[").append(this.zbName);
            if (this.fetchMean) {
                stringBuffer.append("$");
            }
        }
        else {
            stringBuffer.append(this.tableName).append("[").append(this.row).append(",").append(this.col);
        }
        if (this.year != 0) {
            stringBuffer.append(",").append(this.year).append("N");
        }
        if (this.term != 0) {
            stringBuffer.append(",").append(this.term).append(kindIntToSign(this.termKind));
        }
        else if (this.termKind != 0) {
            stringBuffer.append(",").append(kindIntToSign(this.termKind));
        }
        if (this.hasCond) {
            stringBuffer.append(",'").append(this.condition).append("'");
        }
        if (this.dimens != null && !this.dimens.isEmpty()) {
            for (final String s : this.dimens.keySet()) {
                stringBuffer.append(",").append(s).append("=\"").append(this.dimens.get(s)).append("\"");
            }
        }
        if (this.statMode != 0) {
            stringBuffer.append(",").append(statIntToSign(this.statMode));
        }
        stringBuffer.append("]");
        if (!this.taskName.equals("")) {
            stringBuffer.append("@").append(this.taskName);
        }
        return stringBuffer.toString();
    }
    
    @Override
    protected void finalize() throws Throwable {
        --DataCell.DC_LIVE_COUNT;
        super.finalize();
    }
    
    public static void main(final String[] args) {
        final String str1 = "KB01[42,1]";
        final String str2 = "KB01[48,1]";
        System.out.println("str1:" + str1.hashCode() + "," + hash(str1));
        System.out.println("str2:" + str2.hashCode() + "," + hash(str2));
    }
    
    private static int hash(final Object x) {
        int h = x.hashCode();
        h += ~(h << 9);
        h ^= h >>> 14;
        h += h << 4;
        h ^= h >>> 10;
        return h;
    }
}
